/* "eqcs.cc"
 * Copyright (C) 1999 Peter Belkner <pbelkner@berlin.snafu.de>
 *
 * This file is part of the eqcs library. This library is free
 * software; you can redistribute it and/or modify it under the terms of
 * the GNU General Public License as published by the Free Software
 * Foundation; either version 2, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; see the file COPYING.  If not, write to the
 * Free Software Foundation, 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */
#include <stdlib.h>
#include <ctype.h>
#include <iostream>
#include <sstream>
#include "eqcs_qc.h"
#include "eqcs_vedral.h"

using namespace std;

enum { BITS_PER_REGISTER = 8 };

const char prompt[] = "eqcs >";

void usage(ostream &os)
{
	os << "To test the plain adder type an expression like"  << endl;
	os << prompt << "1 + 2" << endl;
	os << "and hit the enter button. Be aware that the sum is ";
	os << "less then " << (1 << BITS_PER_REGISTER) << "." << endl;

	os << "To quit eqcs type 'q' like in"  << endl;
	os << prompt << 'q' << endl;
	os << "and hit the enter button." << endl;

	os << prompt << endl;
	os << prompt << endl;
}

int main(int argc, char **argv)
{
	const int n = BITS_PER_REGISTER;

	// Create a program for adding two three-bit numbers.
	EqcsGateArray program;
	EqcsVedral::plain_adder(program, n);

	// The quantum computer initially prepared in state |0>.
	EqcsQc qc(1ul);

	usage(cout);

	for (;;) {
		char buf[256];
		char ch = 0;

		cout << prompt;
		cin.getline(buf, sizeof buf);

		if (! *buf)
			continue;

		istringstream is(buf);
		is >> ch;
		is.putback(ch);

		if (isdigit(ch)) {
			unsigned long lhs = 0;
			unsigned long rhs = 0;

			if (! (is >> lhs)) {
				cout << "error: number expected" << endl;
				continue;
			}

			if (! (is >> ch) || '+' != ch) {
				cout << "error: '+' expected" << endl;
				continue;
			}

			if (! (is >> rhs)) {
				cout << "error: number expected" << endl;
				continue;
			}

			// Prepare the two input registers.
			EqcsState state((lhs << n) | rhs);
			cout << "preparing the quantum computer in state ";
			cout << nums(state, 3, n) << endl;
			qc.prepare(state);

			cout << "expected result: (1,0)|0>" <<
				'|' << ((lhs + rhs) % (1 << n)) << '>' <<
				'|' << rhs << '>' << endl;

			// Perform the calculation.
			cout << "adding ... " << flush;
			qc.perform(program);
			cout << "done: " << nums(qc.state(), 3, n) << endl;
		}
		else if ('q' == ch)
			break;
		else
			cout << "unexpected command" << endl;
	}
}
